int DRAG_NOTHING = -1;
int DRAG_ROTATE_SCALE = 0;
int DRAG_TRANSLATE = 1;

boolean draw_bounds;

class PlacedImage
{
  PImage img;
  PMatrix2D M;

  PlacedImage( PImage img )
  {
    this.img = img;
    M = new PMatrix2D();
  }

  void drawSelf()
  {
    pushMatrix();
    applyMatrix( M );
    image( img, 0, 0 );

    if ( draw_bounds ) {
      noFill();
      stroke( 255, 255, 0 );
      rect( 0, 0, img.width, img.height );
      ellipse( 0, 0, 10, 10 );
      ellipse( img.width, 0, 10, 10 );
      ellipse( 0, img.height, 10, 10 );
      ellipse( img.width, img.height, 10, 10 );
    }
    popMatrix();
  }

  int hitTest( float x, float y )
  {
    PMatrix2D M2 = new PMatrix2D( M );
    M2.invert();
    
    float rad = 5.0 * sqrt( sq( M2.m00 ) + sq( M2.m01 ) );
    float mx = M2.multX( x, y );
    float my = M2.multY( x, y );

    if ( dist( mx, my, 0, 0 ) < rad ) {
      return DRAG_ROTATE_SCALE;
    }
    if ( dist( mx, my, img.width, 0 ) < rad ) {
      return DRAG_ROTATE_SCALE;
    }
    if ( dist( mx, my, 0, img.height ) < rad ) {
      return DRAG_ROTATE_SCALE;
    }
    if ( dist( mx, my, img.width, img.height ) < rad ) {
      return DRAG_ROTATE_SCALE;
    }

    if ( (mx > 0) && (mx < img.width) && 
      (my > 0) && (my < img.height) ) {
      return DRAG_TRANSLATE;
    }

    return DRAG_NOTHING;
  }

  void drag( int mode, float pmx, float pmy, float mx, float my )
  {
    if ( mode == DRAG_TRANSLATE ) {
      M.preApply( 1, 0, mx - pmx, 0, 1, my - pmy );
    } else if ( mode == DRAG_ROTATE_SCALE ) {
      PMatrix2D T = new PMatrix2D();

      float cx = M.multX( img.width/2.0, img.height/2.0 );
      float cy = M.multY( img.width/2.0, img.height/2.0 );
      float s = dist( cx, cy, mx, my ) / dist( cx, cy, pmx, pmy );
      float a2 = atan2( my - cy, mx - cx );
      float a1 = atan2( pmy - cy, pmx - cx );

      T.translate( cx, cy );
      T.rotate( a2 - a1 );
      T.scale( s );
      T.translate( -cx, -cy );
      M.preApply( T );
    }
  }
}