// Drawing the fractal Koch curve using geometric context.

void setup()
{
  size( 600, 400 );
}

void drawKoch( int levs )
{
  if( levs == 0 ) {
    // If we've decided that this is a base case, just
    // draw a straight line from (0,0) to (100,0).
    line( 0, 0, 100, 0 );
  } else {
    // If this is a recursive case, draw four lines in
    // the standard Koch formation.  But scale the whole
    // thing so that the path made by the four lines 
    // starts at (0,0) and ends at (100,0).  That will
    // allow all the recursive levels to "occupy the
    // same space".
    
    pushMatrix();
    scale( 1.0 / 3.0 );
    
    // Draw the first part of the recursive curve (just
    // a straight horizontal line that reaches 1/3 the
    // length of the parent).
    drawKoch( levs - 1 );
    
    // Draw the second part: shifted to the right and
    // pointing 60 degrees upward.
    pushMatrix();
    translate( 100, 0 );
    rotate( radians( -60 ) );
    drawKoch( levs - 1 );
    popMatrix();
    
    // Draw the third part: shifted more to the right
    // and pointing 60 degrees downward from the 
    // previous segment.
    pushMatrix();
    translate( 200, 0 );
    rotate( radians( 60 ) );
    translate( -100, 0 );
    drawKoch( levs - 1 );
    popMatrix();
    
    // Draw the last part.
    pushMatrix();
    translate( 200, 0 );
    drawKoch( levs - 1 );
    popMatrix();
    
    popMatrix();
  }
}

void draw()
{
  background( 255 );
  strokeWeight( 20.0 );
  pushMatrix();
  translate( width/2, height/2 );
  scale( 5 );
  translate( -50, 0 );
  drawKoch( 4 );
  popMatrix();
}